<?php
/*
     BatchDelete.body.php Mediawiki extension. Batch delete page(s).
     See detail http://metabolomics.jp/wiki/Help:Extension/BachDelete
     Last-modified: 2011.02.21
     Version: 0.1

     *********************************************************************

     Copyright (C) 2008-2011 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/
 
class BatchDelete extends SpecialPage
{
	var $prefix = 'bd_';

	function BatchDelete() {
		SpecialPage::SpecialPage( "BatchDelete" );
		self::loadMessages();
	}
 
	function execute( $par ) {
		global $wgUser, $wgRequest, $wgOut;

		# User check
		if( !$wgUser->isAllowed( 'batchdelete' ) ){
			return $wgOut->permissionRequired( 'batchdelete' );
		}
 
		# preprocess
		$this->setHeaders();

		$this->outputForm();

		# data check
		$pages = $wgRequest->getText( $this->prefix . 'pages' );
		if( isset( $pages ) && strlen( $pages ) > 0 ){
			# delete process
			$reason = $wgRequest->getText( $this->prefix . 'reason' );
			if( !isset( $reason) || strlen( $reason ) == 0 )
				$reason = "";
			$list = split( "\n", $pages );
			$num = count( $list );

			$cSL = 0;
			$cFL = 0;
			$cNL = 0;
			$succeeded_list = "";
			$failed_list = "";
			$notexists_list = "";
			for( $i = 0; $i < $num; $i ++ ){
				$targetTitle = Title::newFromText( $list[$i] );
				if( $targetTitle->exists() ) {
					if( $targetTitle->getNamespace() == NS_IMAGE ) {
						$targetArticle = new ImagePage( $targetTitle );
					} else {
						$targetArticle = new Article( $targetTitle );
					}
					# delete
					$dbw = wfGetDB( DB_MASTER );
					$dbw->begin();
					$success = $targetArticle->doDeleteArticle( $reason );
					$dbw->immediateCommit();
					if( $success == true ) {
						$succeeded_list .= '<li>' . $list[$i] . "</li>\n";
						$cSL ++;
					} else {
						# failed
						$failed_list .= '<li>' . $list[$i] . "</li>\n";
						$cFL ++;
					}
				} else {
					# not exists;
					$notexists_list .= '<li>' . $list[$i] . "</li>\n";
					$cNL ++;
				}
			}
			# result
			$wgOut->addHTML( '<hr><div>' . wfMsg( 'deletedlist' ) . '</div>' );
			$wgOut->addHTML( 'Total: ' . $cSL );
			$wgOut->addHTML( '<ul>' . $succeeded_list . '</ul>' );
			$wgOut->addHTML( '<hr><div>' . wfMsg( 'failedlist' ) . '</div>' );
			$wgOut->addHTML( 'Total: ' . $cFL );
			$wgOut->addHTML( '<ul>' . $failed_list . '</ul>' );
			$wgOut->addHTML( '<hr><div>' . wfMsg( 'notexistslist' ) . '</div>' );
			$wgOut->addHTML( 'Total: ' . $cNL );
			$wgOut->addHTML( '<ul>' . $notexists_list . '</ul>' );
		}
	}

	function outputForm() {
		global $wgOut, $wgArticlePath, $wgTitle;

		$action = str_replace( '$1', '', $wgArticlePath ) . $wgTitle;

		$wgOut->addHTML( '<form action="' . $action . '" method="post">' . "\n" );
		$wgOut->addHTML( ' <div>' . wfMsg( 'pageexplanation' ) . '</div>' . "\n" );
		$wgOut->addHTML( ' ' . wfMsg( 'reason' ) . ' <input type="text" name="' . $this->prefix . 'reason" size="50"><br />' . "\n" );
		$wgOut->addHTML( ' <textarea name="' . $this->prefix . 'pages" rows="6" cols="40"></textarea><br />' . "\n" );
		$wgOut->addHTML( ' <input type="submit" value="delete">' . "\n" );
		$wgOut->addHTML( '</form>' . "\n" );
	}

	function loadMessages() {
		static $messagesLoaded = false;
		global $wgMessageCache;
		if ( !$messagesLoaded ) {
			$messagesLoaded = true;
 
			require( dirname( __FILE__ ) . '/Batchdelete.i18n.php' );
			foreach ( $allMessages as $lang => $langMessages ) {
				$wgMessageCache->addMessages( $langMessages, $lang );
			}
		}
		return true;
	}
}
