<?php
/*
     CreateGraph.php Mediawiki extension. Create and display to graph, powered by jpgraph.
     See detail http://metabolomics.jp/wiki/Help:Extension/CreateGraph
     Last-modified: 2008.10.23
     Version: 0.2
     Change:
       - compliant mediawiki 1.12 or later

     History
     Version: 0.1 2008.09.17

     *********************************************************************

     Copyright (C) 2008 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/

if( !defined( 'MEDIAWIKI' ) ){
	die( 'This file is a MediaWiki extension, see <a href="http://metabolomics.jp/wiki/Help:Extension/CreateGraph">CreateGraph.php</a>.' );
}

$wgExtensionFunctions[] = 'efSetupCreateGraph';
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'CreateGraph',
	'version' => '0.2',
	'author' => 'K, Suwa',
	'description' => 'Create and display to graph, powered by jpgraph',
	'url' => 'http://metabolomics.jp/wiki/Help:Extension/CreateGraph',
);

$wgHooks['LanguageGetMagic'][] = 'efCreateGraphLanguageGetMagic';
 
class CreateGraph {

	/**
	 * create graph
	 * e.g. {{#graph:vbar|size=300x300;title=test;legend=0x0;label=a,b,c;data1=10,34,20}} => [create bar graph]
	 *
	 * @param $parser Parser Parent parser
	 * @param $kind string kind of graph ( line, hbar, vbar, pie, pie3 )
	 * @param $data string data which separated ';'
	 * @return string <graph> tag extension
	*/
	function graph( &$parser, $kind = '', $data )
	{
		$str = '<graph graph="' . $kind . '" data="' . $data . '" />';
		if( method_exists( $parser, 'preprocessToDom' ) ){
			# mw 1.12 or later
			return array( $parser->preprocessToDom( $str ), 'isChildObj' => true );
		}

		# mw 1.11 or under
		return $str;
	}

	function replaceGraphLink( $input, $args, $parser ) {
		global $egScriptPath;

		if( !isset( $args['graph'] ) || !isset( $args['data'] ) ){
			return '<!-- CreateGraph.php - Not found \'graph\' and/or \'data\' param. -->';
		}
		$html = '<img src="' . $egScriptPath . '/graph.php?graph=' . $args['graph'] . '&data=' . $args['data'];

		return $html . '" />';
	}
}

function efSetupCreateGraph() {
	global $wgParser;

	$createGraph = new CreateGraph;

	$wgParser->setFunctionHook( 'graph', array( &$createGraph, 'graph' ) );
	$wgParser->setHook( 'graph', array( &$createGraph, 'replaceGraphLink' ) );
}

function efCreateGraphLanguageGetMagic( &$magicWords, $langCode ) {
	$magicWords['graph'] = array( 0, 'graph' );

	return true;
}

