<?php
/*
     GetEnvironment.php Mediawiki extension. Get php enviroment variables.
     See detail http://metabolomics.jp/wiki/Help:Extension/GetEnvironment
     Last-modified: 2010.04.23
     Version: 0.1

     *********************************************************************

     Copyright (C) 2010 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/
 
if ( !defined( 'MEDIAWIKI' ) ) {
	die( 'This file is a MediaWiki extension, see <a href="http://metabolomics.jp/wiki/Help:Extension/GetEnvironment">GetEnvironment.php</a>.' );
}

$wgExtensionFunctions[] = 'efSetupGetEnvironment';
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'GetEnvironment',
	'version' => '0.1',
	'author' => 'K, Suwa',
	'description' => 'get php enviroment variables.',
	'url' => 'http://metabolomics.jp/wiki/Help:Extension/GetEnvironment',
);

$wgHooks['LanguageGetMagic'][] = 'efGetEnvironmentLanguageGetMagic';
 
class GetEnvFunctions {

	/**
	 * Extracting string
	 * e.g. {{#substring:Hello world|1}} => "ello world"
	 * e.g. {{#substring:Hello world|1|4}} => "ell"
	 *
	 * @param $parser Parser Parent parser
	 * @param $str string Target string
	 * @param $start int Start index (including value)
	 * @param $end int End index (not including value)
	 * @return string Parts string
	 */
	function get( &$parser, $id ) {
		$id = strtolower( $id );
		if( strcmp( $id, "lang" ) != 0 ){
			return "<!-- GetEnvironment.php - Don't supported id. -->";
		}

		$langs = getenv( "HTTP_ACCEPT_LANGUAGE" );
		if( strlen( $langs ) == 0 ){
			return "<!-- GetEnvironment.php - Don't specified language. -->";
		}

		$a_lang = explode( ",", $langs );
		return $a_lang[0];
	}
}

function efSetupGetEnvironment() {
	global $wgParser;

	$geFunctions = new GetEnvFunctions;

	$wgParser->setFunctionHook( 'getenv', array( &$geFunctions, 'get' ) );
}

function efGetEnvironmentLanguageGetMagic( &$magicWords, $langCode ) {
	$magicWords['getenv']    = array( 0, 'getenv' );

	return true;
}
