<?php
/*
     HTMLForm.php Mediawiki extension. Form HTML tags.
     See detail http://metabolomics.jp/wiki/Help:Extension/HTMLForm
     Last-modified: 2009.10.08
     Version: 0.3
     Change:
       - 'encode' accept $code parameter

     Version: 0.2 2008.11.18
     Change:
       - add {{#encode:}}
       - add {{#decode:}}
       - compliant mediawiki 1.12 or later
       - 'tag' extension name rename to 'formtag'
       - bug fix

     History
     Version: 0.1 2008.09.29

     *********************************************************************

     Copyright (C) 2008-2010 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/

if( !defined( 'MEDIAWIKI' ) ){
	die( 'This file is a MediaWiki extension, see <a href="http://metabolomics.jp/wiki/Help:Extension/HTMLForm">HTMLForm.php</a>.' );
}

$wgExtensionFunctions[] = 'efTagFunctionSetup';
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'HTMLForm',
	'version' => '0.3',
	'author' => 'K, Suwa',
	'description' => 'Form HTML tags',
	'url' => 'http://metabolomics.jp/wiki/Help:Extension/HTMLForm',
);

$wgHooks['LanguageGetMagic'][] = 'efTagFunctionsLanguageGetMagic';
 
class TagFunctions
{
	/**
	 * replace AllowedTag
	 * e.g. {{#formtag:input|type="submit"}} => <input type="submit">
	 *      {{#formtag:form|action="-"|...}} => <form action="-" method="post">...</form>
	 *
	 * @param $parser Parser Parent parser
	 * @param $tag string HTML tag
	 * @param $arg string HTML tag parameter
	 * @param $input string data which be nipped HTML tag
	 * @return string which parse by HTMLForm.php 
	*/
	function formtag( &$parser, $tag, $arg = '', $input = '' ) {
		$html = "";

		if( strncmp( $tag, 'input', 6) == 0 ){
			$html = '<input ' . $arg . '>';

		} else if( strncmp( $tag, 'form', 5 ) == 0 || strncmp( $tag, 'textarea', 9 ) == 0 ||
				strncmp( $tag, 'select', 7 ) == 0 || strncmp( $tag, 'option', 7 ) == 0 ||
				strncmp( $tag, 'optgroup', 9 ) == 0 || strncmp( $tag, 'fieldset', 9 ) == 0 ||
				strncmp( $tag, 'legend', 7 ) == 0 || strncmp( $tag, 'label', 6 ) == 0){
			$html = '<' . $tag . ' ' . $arg . '>' . $input . '</' . $tag . '>';
		}

		if( method_exists( $parser, 'preprocessToDom' ) ){
			# mw 1.12 or later
			return array( $parser->preprocessToDom( $html ), 'isChildObj' => true );
		}

		# mw 1.11 or under
		return $html;
	}

	/**
	 * get posted data.
	 * e.g. {{#get:data}} => abcdef
	 *
	 * @param $parser Parser Parent parser
	 * @param $name string data name
	 * @param $sep string separator, if data is array
	 * @return posted data
	 */
	function get( &$parser, $name, $sep = ' ' ) {
		global $wgRequest, $egParamPrefix;
		if( !isset( $egParamPrefix ) )
			$egParamPrefix = "my_";

		$parser->disableCache();

		if( strpos( $name, '[]' ) === FALSE ) {
			$data = $wgRequest->getText( "$egParamPrefix$name" );
		} else {
			$name = str_replace( '[]', '', $name );
			$tmp = $wgRequest->getArray( "$egParamPrefix$name" );
			if( count( $tmp ) > 0 )
				$data = implode( $sep, $tmp );
			else
				$data = "";
		}
		return $data;
	}

	/**
	 * encode string.
	 * e.g. {{#encode:a%b#c!d=e}} => a%25b%23c%21d%3De
	 *
	 * @param $parser Parser Parent parser
	 * @param $str string
         * @param $code character code(current support is only 'EUC-JP')
	 * @return posted encoded string
	 */
	function strEncode( &$parser, $str, $code = '' ) {
		$code2 = "";
		if( strlen( $code ) == 0 ){
			$code = urlencode( $str );
			for( $i = 0; $i < strlen( $code ); $i ++ ){
				$ch = substr( $code, $i, 1 );
				if( strcmp( $ch, "%" ) == 0 ){
					$code2 .= substr( $code, $i, 3 );
					$i += 2;
				} else if( strcmp( $ch, "+" ) == 0 ){
					$code2 .= "%20";
				} else {
					$code2 .= "%" . bin2hex( $ch );
				}
			}
		} else if( strcmp( "EUC-JP", $code ) == 0 ){
			$code2 = urlencode( mb_convert_encoding( $str, "EUC-JP", "UTF-8" ) );
		}

		return $code2;
#		return urlencode( $str );
	}

	/**
	 * decode string.
	 * e.g. {{#decode:{{#encode:{{a%25b%23c%21d%3De}}}} => abcde
	 *
	 * @param $parser Parser Parent parser
	 * @param $str encoded string
	 * @return posted decoded string
	 */
	function strDecode( &$parser, $str ) {
		return urldecode( $str );
	}

	function replaceInput( $input, $args, $parser ) {
		if( !isset( $args['type'] ) ){
			return '<!-- HTMLForm.php - input - Not found \'type\' param. -->';
		}
		global $egParamPrefix;
		if( !isset( $egParamPrefix ) )
			$egParamPrefix = "my_";
		$html = '<input type="' . $args['type'] . '"';
		if( isset( $args['name'] ) ){
			$html .= ' name="' . $egParamPrefix . $args['name'] . '"';
		}
		if( isset( $args['size'] ) ){
			$html .= ' size="' . $args['size'] . '"';
		}
		if( isset( $args['maxlen'] ) ){
			$html .= ' maxlength="' . $args['maxlen'] . '"';
		}
		if( isset( $args['value'] ) ){
			$html .= ' value="' . $args['value'] . '"';
		}
		if( isset( $args['checked'] ) ){
			$html .= ' checked="' . $args['checked'] . '"';
		}
	
		$this->appendParam( $html, $args );
	
		return $html . '>';
	}
	
	function replaceTextarea( $input, $args, $parser ) {
		$this->decodeTag( $input );
	
		if( !isset( $args['name'] ) ){
			return '<!-- HTMLForm.php - textarea - Not found \'name\' param. -->';
		}
	
		global $egParamPrefix;
		if( !isset( $egParamPrefix ) )
			$egParamPrefix = "my_";
		$html = '<textarea name="' . $egParamPrefix . $args['name'] . '"';
		if( isset( $args['cols'] ) ){
			$html .= ' cols="' . $args['cols'] . '"';
		}
		if( isset( $args['rows'] ) ){
			$html .= ' rows="' . $args['rows'] . '"';
		}
	
		$this->appendParam( $html, $args );
	
		return $html . '>' . $input . '</textarea>';
	}
	
	function replaceSelect( $input, $args, $parser ) {
		$this->decodeTag( $input );
	
		if( !isset( $args['name'] ) ){
			return '<!-- HTMLForm.php - select - Not found \'name\' param. -->';
		}
	
		global $egParamPrefix;
		if( !isset( $egParamPrefix ) )
			$egParamPrefix = "my_";
		$html = '<select name="' . $egParamPrefix . $args['name'] . '"';
		if( isset( $args['size'] ) ){
			$html .= ' size="' . $args['size'] . '"';
		}
		if( isset( $args['multiple'] ) ){
			$html .= ' multiple="' . $args['multiple'] . '"';
		}
	
		$this->appendParam( $html, $args );
	
		return $html . '>' . $input . '</select>';
	}
	
	function replaceOption( $input, $args, $parser ) {
		$this->decodeTag( $input );
	
		$html = '<option';
		if( isset( $args['value'] ) ){
			$html .= ' value="' . $args['value'] . '"';
		}
		if( isset( $args['label'] ) ){
			$html .= ' label="' . $args['label'] . '"';
		}
		if( isset( $args['selected'] ) ){
			$html .= ' selected="' . $args['selected'] . '"';
		}
	
		$this->appendParam( $html, $args );
	
		return $html . '>' . $input . '</option>';
	}
	
	function replaceOptgroup( $input, $args, $parser ) {
		$this->decodeTag( $input );
	
		if( !isset( $args['label'] ) ){
			return '<!-- HTMLForm.php - optgroup - Not found \'label\' param. -->';
		}
		$html = '<optgroup label="' . $args['label'] . '"';
	
		$this->appendParam( $html, $args );
	
		return $html . '>' . $input . '</optgroup>';
	}
	
	function replaceFieldset( $input, $args, $parser ) {
		$this->decodeTag( $input );
	
		return '<fieldset>' . $input . '</fieldset>';
	}
	
	function replaceLegend( $input, $args, $parser ) {
		$this->decodeTag( $input );
	
		return '<legend>' . $input . '</legend>';
	}
	
	function replaceLabel( $input, $args, $parser ) {
		$this->decodeTag( $input );
	
		if( !isset( $args['for'] ) ){
			return '<!-- HTMLForm.php - label - Not found \'label\' param. -->';
		}
		$html = '<label for="' . $args['for'] . '"';
	
		return $html . '>' . $input . '</label>';
	}
	
	function replaceForm( $input, $args, $parser ) {
		global $wgScriptPath, $wgArticlePath, $wgParser;
	
		$this->decodeTag( $input );
	
		# exceptional hasty parse
		$input = $wgParser->replaceInternalLinks( $input );
		$input = $wgParser->replaceExternalLinks( $input );
	
		if( !isset( $args['action'] ) ){
			return '<!-- HTMLForm.php - form - Not found \'action\' param. -->';
		}
		$url = $args['action'];
	
//		$pos = mb_strpos( $url, 'http://' );
//		if( $pos !== false ){
		if( ereg( '^[a-zA-Z]*://.*$', $url ) ){
			return '<!-- HTMLForm.php - form - Found external url. Abort parse tag. -->';
		}
	
		$pos = mb_strpos( $url, $wgScriptPath );
		if( $pos === false || $pos != 0 ){
			if( !isset( $wgArticlePath ) )
				return '<!-- HTMLForm.php - form - Found invalid url. Abort parse tag. -->';
			$pos = mb_strpos( $url, str_replace( '$1', '',  $wgArticlePath ) );
		}
		if( $pos === false || $pos != 0 ){
			return '<!-- HTMLForm.php - form - Found invalid url. Abort parse tag. -->';
		}
	
		return '<form action="' . $url . '" method="post">' . $input . '</form>';
	}
	
	function appendParam( &$html, $args ) {
		if( isset( $args['id']) ){
			$html .= ' id="' . $args['id'] . '"';
		}
		if( isset( $args['class'] ) ){
			$html .= ' class="' . $args['class'] . '"';
		}
		if( isset( $args['style'] ) ){
			$html .= ' style="' . $args['style'] . '"';
		}
	}
	
	function decodeTag( &$input ) {
		$allow_tags = array(
			'table',
			't[rdh]',
			'h[1-6]',
			'[bh]r',
			'p',
			'div',
			'span',
			'su[bp]',
			'font',
		);
		$tags = implode( '|', $allow_tags );
	
		$regex1 = "<((\"[^\"]*\"|'[^']'|[^'\">])*)>";
		$regex2 = "&lt;((?:(input|textarea|$tags)) (\"[^\"]*\"*|'[^']*'|[^'\";])*)&gt;";
		$regex3 = "&lt;([/]?(?:$tags)[ ]*[/]?)&gt;";
	
		$input = mbereg_replace( $regex1, "&lt;\\1&gt;", $input );
		$input = mbereg_replace( $regex2, "<\\1>", $input );
		$input = mbereg_replace( $regex3, "<\\1>", $input );
	}
}

function efTagFunctionSetup() {
	global $wgParser;

	$tagFunctions = new TagFunctions;

	$wgParser->setHook( 'input', array( &$tagFunctions, 'replaceInput' ) );
	$wgParser->setHook( 'textarea', array( &$tagFunctions, 'replaceTextarea' ) );
	$wgParser->setHook( 'select', array( &$tagFunctions, 'replaceSelect' ) );
	$wgParser->setHook( 'option', array( &$tagFunctions, 'replaceOption' ) );
	$wgParser->setHook( 'optgroup', array( &$tagFunctions, 'replaceOptgroup' ) );
	$wgParser->setHook( 'fieldset', array( &$tagFunctions, 'replaceFieldset' ) );
	$wgParser->setHook( 'legend', array( &$tagFunctions, 'replaceLegend' ) );
	$wgParser->setHook( 'label', array( &$tagFunctions, 'replaceLabel' ) );
	$wgParser->setHook( 'form', array( &$tagFunctions, 'replaceForm' ) );

	$wgParser->setFunctionHook( 'formtag', array( &$tagFunctions, 'formtag' ) );
	$wgParser->setFunctionHook( 'get', array( &$tagFunctions, 'get' ) );
	$wgParser->setFunctionHook( 'encode', array( &$tagFunctions, 'strEncode' ) );
	$wgParser->setFunctionHook( 'decode', array( &$tagFunctions, 'strDecode' ) );
}

function efTagFunctionsLanguageGetMagic( &$magicWords, $langCode ) {
	$magicWords['formtag'] = array( 0, 'formtag' );
	$magicWords['get']     = array( 0, 'get' );
	$magicWords['encode']  = array( 0, 'encode' );
	$magicWords['decode']  = array( 0, 'decode' );

	return true;
}
