<?php
/*
     This file is based on the original ParserFunctions.php in "Extension:ParserFunctions".
     Modified since 2009/12/09.
     For the original source, see at http://www.mediawiki.org/wiki/Extension:ParserFunctions.

     *********************************************************************

     JDXFunctions.php Mediawiki extension. Parser functions for JDX.
     See detail http://metabolomics.jp/wiki/Help:Extension/JDXFunctions
     Last-modified: 2009.12.09
     Version: 0.1

     *********************************************************************

     Copyright (C) 2008 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/
 
if ( !defined( 'MEDIAWIKI' ) ) {
	die( 'This file is a MediaWiki extension, see <a href="http://metabolomics.jp/wiki/Help:Extension/JDXFunctions">JDXFunctions.php</a>.' );
}

$wgExtensionFunctions[] = 'efSetupJDXFunctions';
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'JDXFunctions',
	'version' => '0.1',
	'author' => 'K, Suwa',
	'description' => 'Parser functions for JDX',
	'url' => 'http://metabolomics.jp/wiki/Help:Extension/JDXFunctions',
);

$wgHooks['LanguageGetMagic'][] = 'efJDXFunctionsLanguageGetMagic';
 
class JDXFunctions {

	/**
	 * Get filesystem path. for version 1.11, use {{filepath:-}} for version 1.12 or later
	 * e.g. {{#jdxfilepath:title}} => "http://www.metabolomics.jp/mediawiki/images/1/11/title"
	 *
	 * @param $parser Parser Parent parser
	 * @param $title page title
	 * @return string URL to file
	 */
	function jdxFilePath( &$parser, $title ) {
		global $wgServer;

		$img = wfFindFile( $title );
		if( !$img )
			return "";
		return $img->getURL();
	}

	/**
	 * Get jdx file plain size.
	 * e.g. {{#jdxfilesize:title}} => 10000
	 *
	 * @param $parser Parser Parent parser
	 * @param $title page title
	 * @return string plain file size
	 */
	function jdxFileSize( &$parser, $title ) {
		$img = wfFindFile( $title );
		if( !$img )
			return 0;
		$url = $img->getURL();
		if( !ereg( ".gz$", $url ) )
			return 0;
		if( strpos( $url, '/../' ) !== false )
			return 0;

		$path = escapeshellarg( $_SERVER['DOCUMENT_ROOT'] . $url );
		$size = `/bin/gzip -l $path | tail -1 | awk '{print $2}'`;

		return $size;
	}
}

function efSetupJDXFunctions() {
	global $wgParser;

	$jdxFunctions = new JDXFunctions;

	$wgParser->setFunctionHook( 'jdxfilepath', array( &$jdxFunctions, 'jdxFilePath' ) );
	$wgParser->setFunctionHook( 'jdxfilesize', array( &$jdxFunctions, 'jdxFileSize' ) );
}

function efJDXFunctionsLanguageGetMagic( &$magicWords, $langCode ) {
	$magicWords['jdxfilepath'] = array( 0, 'jdxfilepath' );
	$magicWords['jdxfilesize'] = array( 0, 'jdxfilesize' );

	return true;
}
