<?php
/*
     JavaApplet.php Mediawiki extension.
     See detail http://metabolomics.jp/wiki/Help:Extension/JavaApplet
     Last-modified: 2011.02.16
     Version: 0.1

     *********************************************************************

     Copyright (C) 2008-2011 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/

if( !defined( 'MEDIAWIKI' ) ){
	die( 'This file is a MediaWiki extension, see <a href="http://metabolomics.jp/wiki/Help:Extension/JavaApplet">JavaApplet.php</a>.' );
}

$wgExtensionFunctions[] = 'efJavaAppletFunctionSetup';
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'JavaApplet',
	'version' => '0.1',
	'author' => 'K, Suwa',
	'description' => 'Java Applet',
	'url' => 'http://metabolomics.jp/wiki/Help:Extension/JavaApplet',
);
$wgHooks['LanguageGetMagic'][] = 'efJavaAppletLanguageGetMagic';
 
class JavaAppletFunctions
{
	function javaApplet( $input, $args, $parser ) {
		$html = '<applet';
		if( isset( $args['codebase'] ) ){
			$html .= ' codebase="' . $args['codebase'] . '"';
		}
		if( isset( $args['code'] ) ){
			$html .= ' code="' . $args['code'] . '"';
		}
		if( isset( $args['archive'] ) ){
			$html .= ' archive="' . $args['archive'] . '"';
		}
		if( isset( $args['width'] ) ){
			$html .= ' width="' . $args['width'] . '"';
		}
		if( isset( $args['height'] ) ){
			$html .= ' height="' . $args['height'] . '"';
		}
	
		$input = str_replace( '<', '&lt;', $input );
		$input = str_replace( '&lt;param', '<param', $input );
		$input = str_replace( '>', '&gt;', $input );
		$input = str_replace( '/&gt;', '/>', $input );
		return $html . '>' . $input . '</applet>';
	}

	/**
	 * java applet with <param>
	 * e.g. {{#javawithparam:Main.class|Main.jar|.|100|100|data=123;id=java}} =>
	 * <object codebase="." code="Main.class" archive="Main.jar" width="100" height="100">
	 * <param name="data" value="123">
	 * <param name="id" value="java">
	 * </object>
	 *
	 * @param $parser Parser Parent parser
	 * @param $class string Main class file
	 * @param $archive string Archive file
	 * @param $codebase string Codebase
	 * @param $width int Applet width
	 * @param $height int Applet height
	 * @param $param string Passed parameter
	 * @return string Java applet HTML string
	 */
	function javaWithParam( &$parser, $class, $archive, $codebase = '.', $width = 100, $height = 100, $param = '' ) {
#		if( strncmp( $param, '', 4) != 0)
		if( strlen( $param ) > 0 )
			$parser->disableCache();
		$output = "<javaapplet codebase=\"$codebase\" code=\"$class\" archive=\"$archive\" width=\"$width\" height=\"$height\">";
		$params = split( ';', $param );
		foreach( $params as $p ){
			$keys = split( '=', $p );
			$output .= " <param name=\"" . $keys[0] . "\" value=\"" . $keys[1] . "\" />\n";
		}
		$output .= "</javaapplet>";

		return array( $output, 'noparse' => false );
	}
}

function efJavaAppletFunctionSetup() {
	global $wgParser;

	$jaFunctions = new JavaAppletFunctions;

	$wgParser->setHook( 'javaapplet', array( &$jaFunctions, 'javaApplet' ) );

	$wgParser->setFunctionHook( 'javawithparam', array( &$jaFunctions, 'javaWithParam' ) );
}


function efJavaAppletLanguageGetMagic( &$magicWords, $langCode ) {
        $magicWords['javawithparam']= array( 0, 'javawithparam' );

	return true;
}
