<?php
/*
     Use MassBank API extension.
     See detail http://metabolomics.jp/wiki/Help:Extension/MassBankAPI
     Last-modified: 2010.08.27
     Version: 0.1

     *********************************************************************

     Copyright (C) 2010-2011 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/

if( !defined( 'MEDIAWIKI' ) ){
        die( 'This file is a MediaWiki extension, see <a href="http://metabolomics.jp/wiki/Help:Extension/MassBankAPI">MassBankAPI.php</a>.' );
}

$wgExtensionFunctions[] = 'efSetupMassBankAPIFunctions';
$wgExtensionCredits['parserhook'][] = array(
        'name' => 'MassBankAPI',
        'version' => '0.1',
        'author' => 'K, Suwa',
        'description' => 'Get mass data from MassBank.jp',
        'url' => 'http://metabolomics.jp/wiki/Help:Extension/MassBankAPI',
);

$wgHooks['LanguageGetMagic'][] = 'efMassBankAPIFunctionsLanguageGetMagic';
 
class MassBankAPIFunctions {

	var $url = 'http://www.massbank.jp/api/services/MassBankAPI?wsdl';

	/**
	 * Output Mass Peaks
	 * e.g. {{#massPeak:KO00001}} => Mass Peak
	 *
	 * @param $parser Parser Parent parser
	 * @param $massbankid target id
	 * @return string Parts string
	 */
	function massPeak( &$parser, $massbankid ) {

		$ids = array( $massbankid );
		$params = array("ids" => $ids );

		$soap = new SoapClient( $this->url );
		try {
			$res = $soap->getPeak( $params );
		} catch (SoapFault $e) {
			return "<!-- Failed: massPeak -->";
		}

		$ret = $res->return;
		$peak = "";
		for ($i = 0 ; $i < count($ret); $i++) {
			$peak .= "NUM_PEAK:" . $ret->numPeaks . "<br />\n";
			for ($j = 0 ; $j < count($ret->mzs); $j++) {
				$mz = $ret->mzs[$j];
				$inte = $ret->intensities[$j];
				$peak .= "mz:$mz,inte:$inte<br />\n";
			}
		}

		return $peak;
	}

	/**
	 * Output Mass record
	 * e.g. {{#massRecordInfo:KO00001}} => Mass Record Information
	 *
	 * @param $parser Parser Parent parser
	 * @param $massbankid target id
	 * @return string Parts string
	 */
	function massRecordInfo( &$parser, $massbankid ) {

		$ids = array( $massbankid );
		$params = array("ids" => $ids );

		$soap = new SoapClient( $this->url );
		try {
			$res = $soap->getRecordInfo( $params );
		} catch (SoapFault $e) {
			return "<!-- Failed: massRecordInfo -->";
		}

		$ret = $res->return;
		$info = "";
		for ($i = 0 ; $i < count($ret); $i++) {
			$info .= str_replace("\n", "<br>\n", $ret->info );
		}

		return $info;
	}
}

function efSetupMassBankAPIFunctions() {
	global $wgParser;

	$massFunctions = new MassBankAPIFunctions;

	$wgParser->setFunctionHook( 'masspeak', array( &$massFunctions, 'massPeak' ) );
	$wgParser->setFunctionHook( 'massrecordinfo', array( &$massFunctions, 'massRecordInfo' ) );
}

function efMassBankAPIFunctionsLanguageGetMagic( &$magicWords, $langCode ) {
	$magicWords['masspeak']     = array( 0, 'masspeak' );
	$magicWords['massrecordinfo'] = array( 0, 'massrecordinfo' );

	return true;
}

