<?php
/*
     MassFunctions.php Mediawiki extension. Mass functions.
     See detail http://metabolomics.jp/wiki/Help:Extension/MassFunctions
     Last-update: 2011.02.10
     Version: 0.2
     Changes:
       correspond to MediaWiki 1.16

     Version: 0.1 2010.03.05

     *********************************************************************

     Copyright (C) 2008 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/
 
if ( !defined( 'MEDIAWIKI' ) ) {
	die( 'This file is a MediaWiki extension, see <a href="http://metabolomics.jp/wiki/Help:Extension/MassFunctions">MassFunctions.php</a>.' );
}

$wgExtensionFunctions[] = 'efSetupMassFunctions';
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'MassFunctions',
	'version' => '0.2',
	'author' => 'K, Suwa',
	'description' => 'Mass functions',
	'url' => 'http://metabolomics.jp/wiki/Help:Extension/MassFunctions',
);

$wgHooks['LanguageGetMagic'][] = 'efMassFunctionsLanguageGetMagic';
 
class MassFunctions {

	var $url = 'http://www.massbank.jp/api/services/MassBankAPI?wsdl';

	/**
	 * Output Mass viewer applet
	 * e.g. {{#masstemplate:KO00001}} => "<applet -></applet>"
	 *
	 * @param $parser Parser Parent parser
	 * @param $keioid target id
	 * @return string Parts string
	 */
	function spectraApplet( ) {
		global $egMassJarPath;

		$argv = func_get_args();
		$num = count( $argv );
		if( $num <= 1 || $num % 2 == 0 )
			return $num;
		$count = ( $num - 1 ) / 2;
		$html = "{{#javawithparam:DisplayAll.class\n|DisplayAll3.jar\n|$egMassJarPath\n|100%\n|" . (200*$count) . "\n|num=" . $count . ";";
		$exFunctions = new ExFunctions();
		for( $i = 1; $i <= $count; $i ++ ){
			$peak = substr( trim( $exFunctions->searchLine( $argv[0], "^peak1=", "Persist", $argv[$i*2-1] ) ), 16 );
			$prec = trim( $exFunctions->searchLine( $argv[0], " PRECURSOR_M/Z ", "Persist", $argv[$i*2-1] ) );
			if( strlen( $prec ) > 0 )
				$prec = substr( $prec, 40 );
			$html .= "id" . $i . "=" . $argv[$i*2-1] . ";name" . $i . "=" . $argv[$i*2] . ";precursor" . $i . "=$prec;peak" . $i . "=$peak;";
		}
		$html = substr( $html, 0, strlen( $html ) - 1 ) . "}}\n\n";
		
		return array( $html, 'noparse' => false );
	}

	/**
	 * Output Mass record
	 * e.g. {{#masstemplate:KO00001}} => "<pre>...</pre>"
	 *
	 * @param $parser Parser Parent parser
	 * @param $keioid target id
	 * @return string Parts string
	 */
	function massRecordInfoWithApp( &$parser, $keioid ) {
		global $egMassJarPath;

		$ids = array( $keioid );
		$params = array("ids" => $ids );

		$soap = new SoapClient( $this->url );
		try {
			$res = $soap->getRecordInfo( $params );
		} catch (SoapFault $e) {
			return "<!-- Failed: massRecordInfo -->";
		}

		$ret = $res->return;
		$info = $ret->info;
		$lines = split( "\n", $info );
		$precursor = "";
		$peak = "";
		$title = "";
		for( $i = 0; $i < count( $lines ); $i ++ ){
			if( preg_match( '/^MS\$FOCUSED_ION: PRECURSOR_M\/Z .*$/', $lines[$i] ) ){
				$precursor = trim( substr( $lines[$i], 30 ) );
			} else if( preg_match( '/^PK\$PEAK: m\/z int. rel.int.$/', $lines[$i] ) ){
				$i ++;
				for( ; $i < count( $lines ); $i ++ ){
					if( preg_match( '/^ .*$/', $lines[$i] ) ){
						$mz = split( ' ', trim( $lines[$i] ) );
						$peak .= $mz[0] . "," . $mz[2] . ":";
					} else
						break;
				}
			} else if( preg_match( '/^RECORD_TITLE:.*$/', $lines[$i] ) ){
				$title = preg_replace( '/;/', ':', trim( substr( $lines[$i], 13 ) ) );
			}
		}
		return array( "<h2>Applet</h2>\n\n{{#javawithparam:DisplayAll.class\n|DisplayAll3.jar\n|$egMassJarPath\n|100%\n|200\n|num=1;\nid1=$keioid;\nname1=$title;\nprecursor1=$precursor;\npeak1=$peak\n}}\n\n<h2>Image</h2>\n\n[[Image:Mass:$keioid.png]]\n\n<pre>\n" . $info . "\n</pre>\n", 'noparse' => false );
	}
}

function efSetupMassFunctions() {
	global $wgParser;

	$massFunctions = new MassFunctions;

	$wgParser->setFunctionHook( 'spectraapplet', array( &$massFunctions, 'spectraApplet' ) );
	$wgParser->setFunctionHook( 'massrecordinfowithapp', array( &$massFunctions, 'massRecordInfoWithApp' ) );
}

function efMassFunctionsLanguageGetMagic( &$magicWords, $langCode ) {
	$magicWords['spectraapplet']     = array( 0, 'spectraapplet' );
	$magicWords['massrecordinfowithapp'] = array( 0, 'massrecordinfowithapp' );

	return true;
}
