<?php
/*
     ScriptFunctions.php Mediawiki extension. Call outside script.
     See detail http://metabolomics.jp/wiki/Help:Extension/ScriptFunctions
     Last-modified: 2010.07.31
     Version: 0.2
     Changes
       - Fix that Can not accept "[[-]]" string in argument.
       - correspond to version 1.12 or later

     Version: 0.1 2008.10.07

     *********************************************************************

     Copyright (C) 2008-2010 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/

 
if ( !defined( 'MEDIAWIKI' ) ) {
	die( 'This file is a MediaWiki extension, see <a href="http://metabolomics.jp/wiki/Help:Extension/ScriptFunctions">ScriptFunctions.php</a>.' );
}

$wgExtensionFunctions[] = 'efSetupScriptFunctions';
$wgExtensionCredits['parserhook'][] = array(
	'name' => 'ScriptFunctions',
	'version' => '0.2',
	'author' => 'K, Suwa',
	'description' => 'Call outside script',
	'url' => 'http://metabolomics.jp/wiki/Help:Extension/ScriptFunctions',
);

$wgHooks['LanguageGetMagic'][] = 'efScriptFunctionsLanguageGetMagic';
 
class ScriptFunctions {
	// try 1 second. try 10 times;
	var $TRY_THRESHOLD = 10;
	var $MAX_FILE_NUM = 10;

	/**
	 * lua script
	 * e.g. {{#lua:print(stdin)|abc}} => abc
	 *
	 * @param $parser Parser Parent parser
	 * @param $src string lua script
	 * @param $data string argument, use as 'stdin' in lua script
	 * @return string result or error
	*/
	function lua( &$parser, $src, $data = '' ) {
		global $egLuaPath, $egTmpPath;
		if( strpos( $src, "dofile" ) !== FALSE )
			return array( "<!-- ScriptFunctions.php - lua - Source contains 'dofile' string. -->", 'noparse' => true );
		if( !isset( $egLuaPath ) )
			return array( "<!-- ScriptFunctions.php - lua - Don't set path to lua binary. -->", 'noparse' => true );
		if( !isset( $egTmpPath ) )
			$egTmpPath = "/tmp";

		$prefix = 'mw_ex_lua_';

		for( $count = 0; $count < $this->TRY_THRESHOLD; $count ++ ){
			$total = 0;
			$dir = opendir(  $egTmpPath );
			if( $dir === FALSE )
				return array( "<!-- ScriptFunctions.php - lua - Read error. -->", "noparse" => true );
		
			$now = time();
			while( ( $fn = readdir( $dir ) ) !== FALSE ){
				$abspath = $egTmpPath . DIRECTORY_SEPARATOR . $fn;
				$name = $fn;
				if( !is_file( $abspath ) )
					continue;
		
				if( strncmp( $name, $prefix, 10 ) == 0 ){
					# target file
					$updatetime = filemtime( $abspath );
					$elapsetime = $now - $updatetime;
					if( $elapsetime >= 10 ) // 10seconds
						unlink( $abspath );
					else
						$total ++;
				}
			}
			closedir( $dir );
		
			if( $total >= $this->MAX_FILE_NUM ){
				usleep( 100000 );	// 100ms
			} else {
				break;
			}
		}
		if( $count >= $this->TRY_THRESHOLD ){
			// failed
			return array( "<!-- ScriptFunctions.php - lua - Don't execute. Server is busy or unknown error. -->", "noparse" => true );
		}
		
		$start  = tempnam( $egTmpPath, $prefix );
		$source = tempnam( $egTmpPath, $prefix );

		$src = str_replace( '&lt;', '<', $src );
		$src = str_replace( '&gt;', '>', $src );
		$src = str_replace( '&amp;', '&', $src );
		$src = str_replace( '&quot;', '"', $src );
		$data = str_replace( '&lt;', '<', $data );
		$data = str_replace( '&gt;', '>', $data );
		$data = str_replace( '&amp;', '&', $data );
		$data = str_replace( '&quot;', '"', $data );

		$fh = fopen( $start, "w" );
		fwrite( $fh, "io = nil; debug = nil; package = nil; os = nil;\nstdin = [=[$data]=]\ndofile( \"$source\" )" );
		fclose( $fh );
		$fh = fopen( $source, "w" );
		fwrite( $fh, $src );
		fclose( $fh );

		$res = "";
		$handle = popen( "cd $egLuaPath; ./lua $start 2>&1", 'r' );
		while( !feof( $handle ) )
			$res .= fgets( $handle, 4096 );
		
		pclose( $handle );

		unlink( $start );
		unlink( $source );

		if( method_exists( $parser, 'preprocessToDom' ) ){
			# mw 1.12 or later
			return array( $parser->preprocessToDom( $res ), 'isChildObj' => true );
		}

		return $res;
	}
}

function efSetupScriptFunctions() {
	global $wgParser;

	$scriptFunctions = new ScriptFunctions;

	$wgParser->setFunctionHook( 'lua', array( &$scriptFunctions, 'lua' ) );
}

function efScriptFunctionsLanguageGetMagic( &$magicWords, $langCode ) {
	$magicWords['lua'] = array( 0, 'lua' );

	return true;
}
