<?php
/*
     YankRemoteImage Mediawiki extension. Yank image on remote server.
     See more YankRemoteImage.setup.php
     See detail http://metabolomics.jp/wiki/Help:Extension/YankRemoteImage

     *********************************************************************

     Copyright (C) 2008-2010 Metabolomics.JP Development Team

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

     *********************************************************************
     Author: Arita laboratory, with Kazuhiro Suwa <metabolome@cb.k.u-tokyo.ac.jp>
     URL: http://metabolomics.jp/
*/

require_once( "HTTP/Request.php" );

global $egYRImap;
$egYRImap = array();

class YankRemoteImage extends SpecialPage
{
	var $prefix = "yri_";

	function YankRemoteImage() {
		SpecialPage::SpecialPage("YankRemoteImage");
		self::loadMessages();
	}
 
	function execute( $par ) {
		global $wgRequest, $wgOut;

		# access control

		# preprocess
		$wgOut->disable();
		wfResetOutputBuffers();
		header( "Content-type: text/plain; charset=utf-8" );

		# output data
		$title = $wgRequest->getText( $this->prefix . "title" );
		if( strlen( $title ) > 0 ){
			$t = Title::makeTitleSafe( NS_IMAGE, $title );
			if( !$t->exists() )
				return;
			$file = wfFindFile( $t );
			if( $file ){
				$path = $file->getURL();
				$width = $file->getWidth();
				$height = $file->getHeight();
			
				$res = '<img alt="' . $t->getNsText() . ':' . $title . '" src="http://' . $_SERVER["SERVER_NAME"] . $path . '" width="' . $width . '" height="' . $height . '" border="0">';
				print $res;
			}
		}
	}

	function loadMessages() {
		static $messagesLoaded = false;
		global $wgMessageCache;
		if ( !$messagesLoaded ) {
			$messagesLoaded = true;
 
			require( dirname( __FILE__ ) . '/YankRemoteImage.i18n.php' );
			foreach ( $allMessages as $lang => $langMessages ) {
				$wgMessageCache->addMessages( $langMessages, $lang );
			}
		}
		return true;
	}

	var $option = array(
		"timeout" => "3",
		"allowRedirects" => false,
		"maxRedirects" => 0,
	);
	function httpAccess( $url, $title ){
		global $wgSitename, $egYankRemoteURL;

		if( strlen( $url ) == 0 )
			return "";

		$http = new HTTP_Request( "http://" . $url . "Special:YankRemoteImage?" . $this->prefix . "title=$title", $this->option );
		$http->addHeader( "User-Agent", "$wgSitename" );
		$http->addHeader( "Referer", "-" );
		$res = $http->sendRequest();
		if( !PEAR::isError( $res ) ) {
			if( $http->getResponseCode() < 400) {
				return $http->getResponseBody();
			}
		}

		return "";
	}
}

function efYankRemoteImageLink( &$parser, &$text, &$strip_state ) {
	global $egYankRemoteURL, $wgCanonicalNamespaceNames, $egYRImap, $wgServerName;

	$yankRemoteImage = new YankRemoteImage();

	// NS_IMAGE -> NS_FILE
	$ns = $wgCanonicalNamespaceNames[NS_IMAGE];
	if( strlen( $ns ) == 0 )
		return true;

	$limit = 50;
	$i = 0;
	$uniq = "yri";
	$own = $wgServerName;
	$ownlength = mb_strlen( $own );
	foreach( $egYankRemoteURL as $key => $url ){
		if( ereg( "[^/]$", $url ) )
			$url .= "/";
		while( $i < $limit && preg_match( "/\[\[[ ]*" . $key . ":$ns:[^\]]*\]\]/i", $text, $matches, PREG_OFFSET_CAPTURE ) != 0 ){
			$str = $uniq . "-" . Parser::getRandomString() . "-" . $uniq;
			$text = preg_replace( "/\[\[[ ]*" . $key . ":$ns:[^\]]*\]\]/i", $str, $text, 1 );
			$page = preg_replace( "/^\[\[[ ]*" . $key . ":$ns:/", "",$matches[0][0] );
			$page = preg_replace( "/\]\][ ]*$/", "", $page );
			if( strncmp( $own, $url, $ownlength ) == 0 ){
				// self server
				$text = preg_replace( "/$str/", "[[$ns:$page]]", $text );
			} else {
				// linkage server
				$change = '<a href="http://' . $url . $ns . ':' . $page . '">' . $yankRemoteImage->httpAccess( $url, $page ) . '</a>';

				$egYRImap["$str"] = $change;
				$i ++;
			}
		}
	}

	return true;
}

function efYankRemoteImageFinish( &$parser, &$text ) {
	global $egYRImap;

	foreach( $egYRImap as $key => $value ){
		$text = preg_replace( "/$key/", $value, $text );
	}

	return true;
}
